/*
* Copyright 2018 Philippe BOUDOT
* www.systemes-embarques.fr
*
* Not for commercial use
*
*/


module FRAME_GEN #(
	parameter pHRES=640,
	parameter pHSS=656,
	parameter pHSE=752,
	parameter pHTOTAL=762,
	
	parameter pVRES=480,
	parameter pVSS=490,
	parameter pVSE=492,
	parameter pVTOTAL=525,
	
	parameter pHSPOL=0,
	parameter pVSPOL=0
 )
 
 (
	input	iCLK_PIX,
	input	iDISP_ENAB,

	output		oHS,
	output reg	oVS,
	output		oDE,

	output [11:0]	oHCNT,
	output [11:0]	oVCNT
 );

	reg [11:0] rHCNT, rVCNT;

	assign oHCNT = rHCNT;
	assign oVCNT = rVCNT;

	assign oDE = (((oVCNT < pVRES) && (oHCNT < pHRES)) && (iDISP_ENAB == 1)) ? 1 : 0;
	assign oHS = ((oHCNT >= pHSS) && (oHCNT < pHSE))  ? ~pHSPOL : pHSPOL;

	always @(posedge iCLK_PIX or negedge iDISP_ENAB)
	begin
		if (iDISP_ENAB == 0) begin
			rHCNT <= 0;
			rVCNT <= 0;
			oVS <= 0;
		end
		else begin
		  rHCNT <= rHCNT+1;
		  
		  if (rHCNT == pHTOTAL-1) begin // Fin de ligne
			rHCNT <=0;
			rVCNT <= rVCNT+1;
			if (rVCNT == pVTOTAL-1) // Fin de trame
			  rVCNT <=0;
		  end
		  
		  if (rHCNT == pHSS-1) begin	// Debut et fin Sync. V sur debut Sync. H
			if (rVCNT == pVSS-1) 
			  oVS <= ~pVSPOL;
			if (rVCNT == pVSE-1) 
			  oVS <= pVSPOL;
		  end
		end
	end

endmodule
