/*
* Copyright 2018 ARDUINO SA (http://www.arduino.cc/)
* This file is part of Vidor IP.
* Copyright (c) 2018
* Authors: Dario Pennisi
*
* This software is released under:
* The GNU General Public License, which covers the main part of 
* Vidor IP
* The terms of this license can be found at:
* https://www.gnu.org/licenses/gpl-3.0.en.html
*
* You can be released from the requirements of the above licenses by purchasing
* a commercial license. Buying such a license is mandatory if you want to modify or
* otherwise use the software for commercial activities involving the Arduino
* software without disclosing the source code of your own applications. To purchase
* a commercial license, send an email to license@arduino.cc.
*
* Adaptation du code par Philippe BOUDOT
* www.systemes-embarques.fr
*/

module MKRVIDOR4000_top
(
  // system signals
  input         iCLK,
  input         iRESETn,
  input         iSAM_INT,
  output        oSAM_INT,
  
  // SDRAM
  output        oSDRAM_CLK,
  output [11:0] oSDRAM_ADDR,
  output [1:0]  oSDRAM_BA,
  output        oSDRAM_CASn,
  output        oSDRAM_CKE,
  output        oSDRAM_CSn,
  inout  [15:0] bSDRAM_DQ,
  output [1:0]  oSDRAM_DQM,
  output        oSDRAM_RASn,
  output        oSDRAM_WEn,

  // SAM D21 PINS
  inout         bMKR_AREF,
  inout  [6:0]  bMKR_A,
  inout  [14:0] bMKR_D,
  
  // Mini PCIe
  inout         bPEX_RST,
  inout         bPEX_PIN6,
  inout         bPEX_PIN8,
  inout         bPEX_PIN10,
  input         iPEX_PIN11,
  inout         bPEX_PIN12,
  input         iPEX_PIN13,
  inout         bPEX_PIN14,
  inout         bPEX_PIN16,
  inout         bPEX_PIN20,
  input         iPEX_PIN23,
  input         iPEX_PIN25,
  inout         bPEX_PIN28,
  inout         bPEX_PIN30,
  input         iPEX_PIN31,
  inout         bPEX_PIN32,
  input         iPEX_PIN33,
  inout         bPEX_PIN42,
  inout         bPEX_PIN44,
  inout         bPEX_PIN45,
  inout         bPEX_PIN46,
  inout         bPEX_PIN47,
  inout         bPEX_PIN48,
  inout         bPEX_PIN49,
  inout         bPEX_PIN51,

  // NINA interface
  inout         bWM_PIO1,
  inout         bWM_PIO2,
  inout         bWM_PIO3,
  inout         bWM_PIO4,
  inout         bWM_PIO5,
  inout         bWM_PIO7,
  inout         bWM_PIO8,
  inout         bWM_PIO18,
  inout         bWM_PIO20,
  inout         bWM_PIO21,
  inout         bWM_PIO27,
  inout         bWM_PIO28,
  inout         bWM_PIO29,
  inout         bWM_PIO31,
  input         iWM_PIO32,
  inout         bWM_PIO34,
  inout         bWM_PIO35,
  inout         bWM_PIO36,
  input         iWM_TX,
  inout         oWM_RX,
  inout         oWM_RESET,

  // HDMI output
  output [2:0]  oHDMI_TX,
  output        oHDMI_CLK,

  inout         bHDMI_SDA,
  inout         bHDMI_SCL,
  
  input         iHDMI_HPD,
  
  // MIPI input
  input  [1:0]  iMIPI_D,
  input         iMIPI_CLK,
  inout         bMIPI_SDA,
  inout         bMIPI_SCL,
  inout  [1:0]  bMIPI_GP,

  // Q-SPI Flash interface
  output        oFLASH_SCK,
  output        oFLASH_CS,
  inout         oFLASH_MOSI,
  inout         iFLASH_MISO,
  inout         oFLASH_HOLD,
  inout         oFLASH_WP

);

// signal declaration

wire        wOSC_CLK;

wire [7:0]  wDVI_RED,wDVI_GRN,wDVI_BLU;
wire        wDVI_HS, wDVI_VS, wDVI_DE;
wire			wDVI_OUT_ENABLE;

wire        wVID_CLK, wVID_CLKx5;
wire			wVID_PLL_LOCK;

wire [11:0]	wVID_X,wVID_Y;

assign wDVI_OUT_ENABLE = iHDMI_HPD && wVID_PLL_LOCK;

// internal oscillator
cyclone10lp_oscillator   osc
  ( 
  .clkout(wOSC_CLK),
  .oscena(1'b1));

FRAME_GEN FRAME_GEN_inst
(
	.iCLK_PIX(wVID_CLK),
	.iDISP_ENAB(wDVI_OUT_ENABLE),

	.oHS(wDVI_HS),
	.oVS(wDVI_VS),
	.oDE(wDVI_DE),

	.oHCNT(wVID_X),
	.oVCNT(wVID_Y)
 );

  
VIDEO_BACKGROUND VIDEO_BACKGROUND_inst 
(
	.iCLK_PIX(wVID_CLK),
	.iPIXEL_X(wVID_X),
	.iPIXEL_Y(wVID_Y),
	
	.oRED(wDVI_RED),
	.oGRN(wDVI_GRN),
	.oBLU(wDVI_BLU)
);
 
DVI_OUT DVIOUT_inst(
	.iPCLK(wVID_CLK),
	.iSCLK(wVID_CLKx5),
	.iRED(wDVI_RED),
	.iGRN(wDVI_GRN),
	.iBLU(wDVI_BLU),
	.iHS(wDVI_HS),
	.iVS(wDVI_VS),
	.iDE(wDVI_DE),
	
	// Attention à l'inversion des signaux pour oHDMI_TX
	.oDVI_DATA({oHDMI_TX[0],oHDMI_TX[1],oHDMI_TX[2]}),
	.oDVI_CLK(oHDMI_CLK)
	
);
 
PLL_VIDEO PLL_VIDEO_inst(
  .areset(1'b0),
  .inclk0(iCLK),
  .c1(wVID_CLK),
  .c2(wVID_CLKx5),
   
  .locked(wVID_PLL_LOCK));



endmodule
